unit CompInstallTools;

interface

uses Classes, SysUtils;

type
  // Type of package to be created.
  TPackagePersonality = (ppDelphi, ppCppBuilder);

  // Callback event when file name was found in path
  TFileNameInPathEvent = procedure(const FileName, Path: string) of object;

  // Error class for this wizard.
  EComponentInstallationError = class(Exception);

// Returns true if current personality is 'CPlusPlusBuilder.Personality'.
function IsCppPersonality: Boolean;

// Returns true if current personality is 'Delphi.Personality'.
function IsDelphiPersonality: Boolean;

// Returns true if APersonality is installed.
function HasPersonality(APersonality: string): Boolean;

// Resolves a path which can contain an environment variable in the form
// $(xyz), e.g. '$(BDS)\lib\debug'.
function ResolveBDSPath(const Path: string): string;

// Decode a path like 'C:\;D:\;D:\Source;C:\BDS\4.0\bin' into single
// directories
procedure DecodePath(Path: string; Directories: TStringList);

// Calls ErrorProc if one of the file names in FileNames can be
// found in Path. Path is of the form 'dir1;dir2;dir3'.
procedure CheckFileNamesInPath(FileNames: TStrings; const Path: string;
  ErrorProc: TFileNameInPathEvent);

// Finds first file name in the path given by Path. FileName will be stripped
// of its directory, and Path is in the form 'dir1;dir2;dir3'.
function FindFirstFileNameInPath(const FileName, Path: string;
  var Found: string): Boolean;

implementation

uses SysConst, StrUtils, CompInstallConsts, ToolsAPI;

// This only works if the BDS is running, and the code is started by it,
// since otherwise the environment variables are not defined.
// IOW, it can't be used for standalone programs.
function ResolveBDSPath(const Path: string): string;
var
  P1, P2: Integer;
begin
  Result := Path;
  P1 := Pos('$(', Path);
  if P1 = 0 then
    Exit;
  P2 := Pos(')', Path);
  if P2 = 0 then
    raise EComponentInstallationError.Create(SInvalidEnvVar);
  Result := Copy(Path, 1, P1 - 1) +
            GetEnvironmentVariable(Copy(Path, P1 + 2, P2 - P1 - 2)) +
            Copy(Path, P2 + 1, MaxInt);
end;

procedure DecodePath(Path: string; Directories: TStringList);
begin
  Directories.Clear;
  if Path = '' then
    Exit;
  Directories.Delimiter := ';';
  Directories.StrictDelimiter := True;
  Directories.DelimitedText := Path;
end;

procedure CheckFileNamesInPath(FileNames: TStrings; const Path: string;
  ErrorProc: TFileNameInPathEvent);
var
  FileName: string;
  Result: string;
begin
  for FileName in FileNames do
    if FindFirstFileNameInPath(FileName, Path, Result) then
      ErrorProc(FileName, Path);
end;

function FindFirstFileNameInPath(const FileName, Path: string; var Found: string): Boolean;
var
  Directories: TStringList;
  Directory: string;
  Name: string;
begin
  Result := False;
  Name := ExtractFileName(FileName);
  Directories := TStringList.Create;
  try
    DecodePath(Path, Directories);
    for Directory in Directories do
      if FileExists(Directory + '\' + Name) then
      begin
        Found := Directory + '\' + Name;
        Result := True;
        Exit;
      end;
  finally
    Directories.Free;
  end;
end;

function IsCppPersonality: Boolean;
begin
  Result := (BorlandIDEServices as IOTAPersonalityServices).CurrentPersonality = sCBuilderPersonality;
end;

function IsDelphiPersonality: Boolean;
begin
  Result := (BorlandIDEServices as IOTAPersonalityServices).CurrentPersonality = sDelphiPersonality;
end;

function HasPersonality(APersonality: string): Boolean;
{$IF RTLVersion < 19.0}
var
  PServices: IOTAPersonalityServices;
  I: Integer;
begin
  PServices := BorlandIDEServices as IOTAPersonalityServices;
  Result := True;
  for I := 0 to PServices.PersonalityCount - 1 do
    if PServices.Personalities[I] = APersonality then
      Exit;
  Result := False;
end;
{$ELSE}
begin
  Result := (BorlandIDEServices as IOTAPersonalityServices).PersonalityExists(APersonality);
end;
{$IFEND}

end.
