unit CompInstallWizard;

interface

procedure Register;

implementation

uses
  Windows, ToolsAPI, SysUtils, Classes, Dialogs, Menus, Controls,
  ActnList, Graphics, Forms,
{$IF RTLVersion >= 19.0}
  DCCStrs,
{$IFEND}
  CompInstallConsts, CompInstallTools, CompInstallPackageBase,
  CompInstallNewPackageFrames, CompInstallExistingPackageFrames,
  CompInstallWizardDlg, OTADebugHelpers, IniFiles;

type
  TCompInstallWizard = class(TInterfacedObject, IOTAWizard, IOTANotifier)
  private
    FInstallItem: TMenuItem;
    FSeparatorItem: TMenuItem;
    FInstallAction: TAction;
    FPersonality: TPackagePersonality;
    function CreateProject(const PackageName: string): IOTAProject;
    function FindProject(const PackageName: string): IOTAProject;
    function OpenProject(const PackageName: string): IOTAProject;
    procedure AddMenuItems;
    procedure RemoveMenuItems;
    procedure ExpertItemClick(Sender: TObject);
    function ContainsFile(InstallProject: IOTAProject; const FileName: string): Boolean;
    procedure AddReferenceFiles(InstallProject: IOTAProject; const FileNames: array of string);
    procedure AddSourceFile(InstallProject: IOTAProject; const SourceName: string);
    procedure AddSourceFiles(InstallProject: IOTAProject; const FileNames: TStrings);
    procedure SetPackageDescription(InstallProject: IOTAProject; const Description: string);
    procedure InstallPackage(InstallProject: IOTAProject);
    procedure CreateAction(ImgIndex: Integer; Services: INTAServices);
    procedure UpdateSearchPath(InstallProject: IOTAProject; const PasName: string);
  public
    constructor Create;
    destructor Destroy; override;

    { IOTAWizard }
    procedure Execute;
    function GetIDString: string;
    function GetName: string;
    function GetState: TWizardState;

    { IOTANotifier }
    procedure AfterSave;
    procedure BeforeSave;
    procedure Destroyed;
    procedure Modified;
  end;

  TPackageCreator = class(TInterfacedObject,
      IOTACreator,
      IOTAProjectCreator,
      IOTAProjectCreator50,
      IOTAProjectCreator80)
  private
    FPackageName: string;
  public
    constructor Create(const PackageName: string);
    function GetCreatorType: string;
    function GetExisting: Boolean;
    function GetFileName: string;
    function GetFileSystem: string;
    function GetOptionFileName: string; // deprecated
    function GetOwner: IOTAModule;
    function GetProjectPersonality: string;
    function GetShowSource: Boolean;
    function GetUnnamed: Boolean;
    function NewOptionSource(const ProjectName: string): IOTAFile; // deprecated
    function NewProjectSource(const ProjectName: string): IOTAFile;
    procedure NewDefaultModule; // deprecated
    procedure NewDefaultProjectModule(const Project: IOTAProject);
    procedure NewProjectResource(const Project: IOTAProject);
  end;

{$IF (RTLVersion >= 19.0) or defined(VER185)}
  TCppPackageCreator = class(TInterfacedObject,
      IOTACreator,
      IOTAProjectCreator,
      IOTAProjectCreator50,
      IOTAProjectCreator80)
  private
    FPackageName: string;
  public
    constructor Create(const PackageName: string);
    function GetCreatorType: string;
    function GetExisting: Boolean;
    function GetFileName: string;
    function GetFileSystem: string;
    function GetOptionFileName: string; // deprecated
    function GetOwner: IOTAModule;
    function GetProjectPersonality: string;
    function GetShowSource: Boolean;
    function GetUnnamed: Boolean;
    function NewOptionSource(const ProjectName: string): IOTAFile; // deprecated
    function NewProjectSource(const ProjectName: string): IOTAFile;
    procedure NewDefaultModule; // deprecated
    procedure NewDefaultProjectModule(const Project: IOTAProject);
    procedure NewProjectResource(const Project: IOTAProject);
  end;

  TCppOTAFile = class(TInterfacedObject, IOTAFile)
  public
    function GetAge: TDateTime;
    function GetSource: string;
  end;
{$IFEND}

procedure Register;
begin
  RegisterPackageWizard(TCompInstallWizard.Create);
end;

// Delphi 2006 does not have a function named TaskMessageDlg.
// Delphi 2006 and 2007 have the same RTLVersion and CompilerVersion, 18.0, but
// Delphi 2007 defines both VER180 and VER185, to distinguish it from Delphi 2006.
{$IF defined(VER180) and not defined(VER185)}
function TaskMessageDlg(const Title, Msg: string; DlgType: TMsgDlgType; Buttons: TMsgDlgButtons;
  HelpCtx: Integer): Integer;
var
  MsgDlg: TForm;
begin
  MsgDlg := CreateMessageDialog(Msg, DlgType, Buttons);
  try
    MsgDlg.Caption := Title;
    Result := MsgDlg.ShowModal;
  finally
    MsgDlg.Free;
  end;
end;
{$IFEND}

{ TCompInstallWizard }

procedure TCompInstallWizard.AddReferenceFiles(InstallProject: IOTAProject;
  const FileNames: array of string);
var
  ReferenceFile: string;
begin
  WriteDebugMessage('AddReferenceFiles');
  for ReferenceFile in FileNames do
    if not ContainsFile(InstallProject, ReferenceFile) then
      InstallProject.AddFile(ReferenceFile, False);
end;

procedure TCompInstallWizard.AddMenuItems;
var
  MainMenu: TMainMenu;
  ParentMenu: TMenuItem;
  I: Integer;
  InsertPos: Integer;
  ImgIndex: Integer;
  Services: INTAServices;
  Bitmap: TBitmap;
begin
  inherited;
  Services := BorlandIDEServices as INTAServices;
  MainMenu := Services.MainMenu;

  // Use default position to start with.
  ParentMenu := MainMenu.Items[CDefaultPos];
  for I := 0 to MainMenu.Items.Count - 1 do
    if CompareText(MainMenu.Items[I].name, SComponentMenuName) = 0 then
    begin
      ParentMenu := MainMenu.Items[I];
      Break;
    end;
  InsertPos := ParentMenu.Count;
  for I := 0 to ParentMenu.Count - 1 do
    if CompareText(ParentMenu.Items[I].name, SComponentMenuSepName) = 0 then
    begin
      InsertPos := I;
      Break;
    end;

  FSeparatorItem := Menus.NewLine;
  ParentMenu.Insert(InsertPos, FSeparatorItem);

  Bitmap := TBitmap.Create;
  try
    Bitmap.LoadFromResourceName(HInstance, SMenuBitmapResourceName);
    ImgIndex := Services.AddMasked(Bitmap, Bitmap.TransparentColor, SWizardMenuItemName);
  finally
    Bitmap.Free;
  end;
  CreateAction(ImgIndex, Services);
  FInstallItem := TMenuItem.Create(nil);
  FInstallItem.Action := FInstallAction;
  ParentMenu.Insert(InsertPos, FInstallItem);
end;

procedure TCompInstallWizard.AddSourceFile(InstallProject: IOTAProject; const SourceName: string);
var
  DcrName: string;
  FileExt: string;
begin
  if not ContainsFile(InstallProject, ExtractFileName(SourceName)) then
  begin
    if FileExists(SourceName) then
    begin
      // .dcr file must be installed *before* corresponding .pas file
      DcrName := ChangeFileExt(SourceName, '.dcr'); // Do not localize!
      if FileExists(DcrName) then
        InstallProject.AddFile(DcrName, False);
      FileExt := ExtractFileExt(SourceName);
      InstallProject.AddFile(SourceName, (FileExt = '.pas') or (FileExt = '.cpp'));
      UpdateSearchPath(InstallProject, SourceName);
    end;
  end;
end;

procedure TCompInstallWizard.AddSourceFiles(InstallProject: IOTAProject; const FileNames: TStrings);
var
  SourceFile: string;
begin
  for SourceFile in FileNames do
    AddSourceFile(InstallProject, SourceFile);
end;

procedure TCompInstallWizard.AfterSave;
begin
  // nothing
end;

procedure TCompInstallWizard.BeforeSave;
begin
  // nothing
end;

function TCompInstallWizard.ContainsFile(InstallProject: IOTAProject;
  const FileName: string): Boolean;
var
  I: Integer;
  ModuleInfo: IOTAModuleInfo;
begin
  Result := True;
  for I := 0 to InstallProject.GetModuleCount - 1 do
  begin
    ModuleInfo := InstallProject.GetModule(I);
    if ModuleInfo.FileName = FileName then
      Exit;
  end;
  Result := False;
end;

constructor TCompInstallWizard.Create;
begin
  inherited;
  AddMenuItems;
  BinDirectory :=
    IncludeTrailingPathDelimiter((BorlandIDEServices as IOTAServices).GetBinDirectory);
  FPersonality := ppDelphi;
end;

// Creates new project and returns its IOTAProject interface.
function TCompInstallWizard.CreateProject(const PackageName: string): IOTAProject;
begin
  if FPersonality = ppCppBuilder then
{$IF (RTLVersion < 19.0) and not defined(VER185)}
    raise EComponentInstallationError.Create(SCantCreateCppBuilderPackage)
{$ELSE}
    Result := (BorlandIDEServices as IOTAModuleServices).CreateModule(
      TCppPackageCreator.Create(PackageName) as IOTACreator) as IOTAProject
{$IFEND}
  else
    Result := (BorlandIDEServices as IOTAModuleServices).CreateModule(
      TPackageCreator.Create(PackageName) as IOTACreator) as IOTAProject;
end;

destructor TCompInstallWizard.Destroy;
begin
  RemoveMenuItems;
  inherited;
end;

procedure TCompInstallWizard.Destroyed;
begin
  // nothing
end;

procedure TCompInstallWizard.Execute;
begin
  // nothing
end;

procedure TCompInstallWizard.ExpertItemClick(Sender: TObject);
var
  InstallPage: TInstallBasePage;
  InstallProject: IOTAProject;
  InstallDlg: TInstallWizardDialog;
begin
  InstallDlg := TInstallWizardDialog.Create(nil);
  try
    if InstallDlg.Execute then
    begin
      InstallPage := InstallDlg.InstallPage;
      if InstallDlg.ExistingPackage then
        InstallProject := OpenProject(InstallPage.PackageName)
      else
      begin
        FPersonality := InstallDlg.Personality;
        InstallProject := CreateProject(InstallPage.PackageName);
      end;
      if Assigned(InstallProject) then
      begin
        AddSourceFiles(InstallProject, InstallDlg.Units);

        // This installs only the usual range of packages for a component
        // file. The user will have to install the rest, if these
        // do not cover all units used by the components.
        if FPersonality = ppCppBuilder then
          AddReferenceFiles(InstallProject,
            ['rtl.bpi', 'designide.bpi', 'vcl.bpi', 'vclactnband.bpi',
             'vclx.bpi', 'xmlrtl.bpi'])
        else
          AddReferenceFiles(InstallProject,
            ['rtl.dcp', 'designide.dcp', 'vcl.dcp', 'vclactnband.dcp',
             'vclx.dcp', 'xmlrtl.dcp']);

        SetPackageDescription(InstallProject, InstallPage.Description);

{$IF RTLVersion >= 18.0} // Installing not (yet) supported in Delphi versions below 2006.
        InstallPackage(InstallProject);
{$IFEND}
      end;
    end;
  finally
    InstallDlg.Free;
  end;
end;

// Finds project if open, and returns its IOTAProject interface.
function TCompInstallWizard.FindProject(const PackageName: string): IOTAProject;
const
  Extensions: array[0..2] of string = ('.bdsproj', '.cbproj', '.dproj'); // Do not translate!
var
  ModuleServices: IOTAModuleServices;
  Ext: string;
begin
  ModuleServices := BorlandIDEServices as IOTAModuleServices;
  for Ext in Extensions do
  begin
    Result := ModuleServices.FindModule(ChangeFileExt(PackageName, Ext)) as IOTAProject;
    if Assigned(Result) then
      Break;
  end;
end;

function TCompInstallWizard.GetIDString: string;
begin
  Result := SWizardIDString;
end;

function TCompInstallWizard.GetName: string;
begin
  Result := SWizardName;
end;

function TCompInstallWizard.GetState: TWizardState;
begin
  Result := [wsEnabled];
end;

procedure TCompInstallWizard.InstallPackage(InstallProject: IOTAProject);

type
  TPackageInstaller = procedure(Self: Integer; FileName: string);

  TStub = packed record
    case Byte of
      0: (LongAdd: Byte;        // Must be $05
          LongOffset: Integer);
      1: (ShortAdd: Word;       // Must be $C083
          ShortOffset: Shortint);
  end;
  PStub = ^TStub;

  TInterface = packed record
    QueryInterface: Pointer;
    AddRef: Pointer;
    Release: Pointer;
    Method: PStub;              // First user method
  end;
  PInterface = ^TInterface;
  PPInterface = ^PInterface;

var
  Builder: IOTAProjectBuilder;
  Options: IOTAProjectOptions;
  Services: IOTAPackageServices;
  Module: HMODULE;
  InstallProcedure: TPackageInstaller;
  Stub: PStub;
  BplFullName, BplFileName, Msg, CompStr: string;
  Offset, Count, PkgNum, CompNum: Integer;
  IDEPackage: string;
  InstallProcName: string;
begin
  Options := InstallProject.GetProjectOptions;
  BplFullName := Options.TargetName;
  BplFileName := ExtractFileName(BplFullName);

  // Establish offset to find base address of TProjectOptions

  // The usual interface method stub is
  //   add eax,longoffset   -- e.g. $05 $FFFFFFF8
  //   jmp methodaddress    -- e.g. $E9 $00001234
  // the stub used in delphicoreide is
  //   add eax,shortoffset  -- e.g. $83 $C0 $F8
  //   jmp methodaddress    -- e.g. $E9 $00001234

  Stub := PPInterface(Options)^^.Method;
  if Stub^.LongAdd = $05 then
    Offset := Stub^.LongOffset
  else if Stub^.ShortAdd = $C083 then
    Offset := Stub^.ShortOffset
  else
    raise EComponentInstallationError.Create(SProjectOptionsNotFound);

  // Find if components were already registered in package.
  Count := 0;
  Services := BorlandIDEServices as IOTAPackageServices;
  for PkgNum := 0 to Services.PackageCount - 1 do
    if Services.PackageNames[PkgNum] = BplFileName then
    begin
      Count := Services.ComponentCount[PkgNum];
      Break;
    end;

  if FPersonality = ppCppBuilder then
  begin
    IDEPackage := SBcbIDE;
    InstallProcName := SCppInstallProcName;
  end
  else
  begin
    IDEPackage := SDelphiCoreIDE;
    InstallProcName := SInstallProcName;
  end;

  // Build package and install it
  Builder := InstallProject.GetProjectBuilder;
  if Builder.BuildProject(cmOTAMake, False, True) then
  begin
    // Find TProjectOptions.InstallPackage method in delphicoreidexxx.bpl.
    Module := LoadPackage(BinDirectory + IDEPackage);
    if Module <> 0 then
    begin
      InstallProcedure := GetProcAddress(Module, PAnsiChar(AnsiString(InstallProcName)));

      // Call TProjectOptions.InstallPackage(BplFullName)
      // Address in interface + offset = address of TProjectOptions instance,
      // to be passed as Self instance pointer for method.
      InstallProcedure(Integer(Options) + Offset, BplFullName);
      UnloadPackage(Module);
    end
    else
      raise EComponentInstallationError.CreateFmt(SDelphiCoreNotFound, [IDEPackage]);

    // Add any new components to message string.
    // If there were already components in the package, the IDE will
    // issue a message the component palette has been updated, etc., so
    // the wizard only has to issue a message if Components.Count = 0
    if Count = 0 then
    begin
      Msg := Format(SPackageInstalled, [BplFullName]);

      // Find any components in the package
      for PkgNum := 0 to Services.PackageCount - 1 do
        if Services.PackageNames[PkgNum] = ExtractFileName(BplFullName) then
        begin
          for CompNum := 0 to Services.ComponentCount[PkgNum] - 1 do
            CompStr := CompStr +
              Services.ComponentNames[PkgNum, CompNum] + ', ';
          Break;
        end;

      // Update message if there were components.
      if CompStr <> '' then
      begin
        SetLength(CompStr, Length(CompStr) - Length(', '));
        Msg := Msg + SComponentsRegistered + CompStr + '.';
      end;

      // Issue message
      TaskMessageDlg(SExpertName, Msg, mtInformation, [mbOK], 0);
    end;
  end
  else
    TaskMessageDlg(SExpertName, Format(SErrorBuilding, [BplFullName]), mtError,
      [mbOK], 0);

end;

procedure TCompInstallWizard.Modified;
begin
  // nothing;
end;

procedure TCompInstallWizard.UpdateSearchPath(InstallProject: IOTAProject;
  const PasName: string);
var
  Directories: TStringList;
  PasDirectory: string;
  Options: IOTAProjectOptions;
{$IF RTLVersion >= 19.0} // Delphi 2009 and above
  BaseConfig: IOTABuildConfiguration;
{$IFEND}
  SearchPath: string;
begin
  PasDirectory := ExcludeTrailingPathDelimiter(ExtractFilePath(PasName));
  Options := InstallProject.ProjectOptions;
{$IF RTLVersion >= 19.0}
  BaseConfig := (Options as IOTAProjectOptionsConfigurations).GetBaseConfiguration;
  SearchPath := BaseConfig.Value[SUnitSearchPath]; // 'DCC_UnitSearchPath'
{$ELSEIF RTLVersion >= 18.0} // Delphi 2006, 2007
  SearchPath := Options.Values[SUnitSearchPath]; // 'UnitDir'
{$IFEND}
  Directories := TStringList.Create;
  try
    DecodePath(SearchPath, Directories);
    if Directories.IndexOf(PasDirectory) < 0 then
{$IF RTLVersion >= 19.0}
      BaseConfig.Value[SUnitSearchPath] := SearchPath + ';' + PasDirectory;
{$ELSE}
      Options.Values[SUnitSearchPath] := SearchPath + ';' + PasDirectory;
{$IFEND}
  finally
    Directories.Free;
  end;
end;

procedure TCompInstallWizard.CreateAction(ImgIndex: Integer; Services:
  INTAServices);
begin
  FInstallAction := TAction.Create(nil);
  FInstallAction.ActionList := Services.ActionList;
  FInstallAction.Caption := SMenuItemCaption;
  FInstallAction.Hint := SMenuItemHint;
  FInstallAction.ImageIndex := ImgIndex;
  FInstallAction.OnExecute := ExpertItemClick;
end;

// Opens existing project, if it is not already open.
// Returns IOTAProject interface of the project.

function TCompInstallWizard.OpenProject(const PackageName: string): IOTAProject;
var
  ActionServices: IOTAActionServices;
  Extension: string;
begin
  // If project is already open in IDE, don't open it again.
  Result := FindProject(PackageName);
  if not Assigned(Result) then
  begin
    if Supports(BorlandIDEServices, IOTAActionServices, ActionServices) then
    begin
      if FileExists(ChangeFileExt(PackageName, '.bdsproj')) then
        Extension := '.bdsproj'
      else if FileExists(ChangeFileExt(PackageName, '.dproj')) then
        Extension := '.dproj'
      else if FileExists(ChangeFileExt(PackageName, '.cbproj')) then
        Extension := '.cbproj'
      else if FileExists(ChangeFileExt(PackageName, '.bpk')) then
        Extension := '.bpk'
      else
        Extension := '.dpk';
      ActionServices.OpenProject(ChangeFileExt(PackageName, Extension), True);
      Result := FindProject(PackageName);
    end;
  end;
  if Result.Personality = sCBuilderPersonality then
    FPersonality := ppCppBuilder
  else if Result.Personality = sDelphiPersonality then
    FPersonality := ppDelphi
  else
    raise EComponentInstallationError.Create('Invalid project type');
  OTADebugHelpers.DebugShowOptionValues(Result.ProjectOptions);
end;

procedure TCompInstallWizard.RemoveMenuItems;
begin
  FInstallAction.Free;
  FInstallItem.Free;
  FSeparatorItem.Free;
end;

// Sets package description, if Description <> ''
procedure TCompInstallWizard.SetPackageDescription(InstallProject: IOTAProject;
  const Description: string);
var
  Options: IOTAProjectOptions;
begin
  Options := InstallProject.ProjectOptions;
  if Description <> '' then
    if FPersonality = ppCppBuilder then
      Options.Values[SCppDescriptionOptionValue] := Description
    else
      Options.Values[SPasDescriptionOptionValue] := Description;
  Options.Values[SAppFileExtOptionValue] := 'bpl'; // Do not translate!
end;

{ TPackageCreator }

constructor TPackageCreator.Create(const PackageName: string);
begin
  // Note: this can *not* be '.bdsproj', '.dproj' or '.cbproj'!
  FPackageName := ExpandFileName(ChangeFileExt(PackageName, '.dpk')); // Do not translate!
end;

function TPackageCreator.GetCreatorType: string;
begin
  Result := ToolsAPI.sPackage;
end;

function TPackageCreator.GetExisting: Boolean;
begin
  Result := False;
end;

function TPackageCreator.GetFileName: string;
begin
  Result := FPackageName;
end;

function TPackageCreator.GetFileSystem: string;
begin
  Result := '';
end;

function TPackageCreator.GetOptionFileName: string;
begin
  Result := '';
end;

function TPackageCreator.GetOwner: IOTAModule;
var
  Services: IOTAModuleServices;
begin
  Services := (BorlandIDEServices as IOTAModuleServices);
  Result := Services.MainProjectGroup;
end;

function TPackageCreator.GetProjectPersonality: string;
begin
  Result := sDelphiPersonality;
end;

function TPackageCreator.GetShowSource: Boolean;
begin
  Result := True;
end;

function TPackageCreator.GetUnnamed: Boolean;
begin
  Result := True;
end;

procedure TPackageCreator.NewDefaultModule;
begin
  // Do nothing, deprecated.
end;

procedure TPackageCreator.NewDefaultProjectModule(const Project: IOTAProject);
begin
  // Do nothing.
end;

function TPackageCreator.NewOptionSource(const ProjectName: string): IOTAFile;
begin
  Result := nil; // BCB only.
end;

procedure TPackageCreator.NewProjectResource(const Project: IOTAProject);
begin
  // Do nothing.
end;

function TPackageCreator.NewProjectSource(const ProjectName: string): IOTAFile;
begin
  Result := nil;
end;

{$IF (RTLVersion >= 19.0) or defined(VER185)}
{ TCppPackageCreator }

constructor TCppPackageCreator.Create(const PackageName: string);
begin
  OTADebugHelpers.WriteDebugMessage('Creating ' + PackageName);
  FPackageName := ExpandFileName(ChangeFileExt(PackageName, '.cpp'));
end;

function TCppPackageCreator.GetCreatorType: string;
begin
  Result := sCppPackage;
  WriteDebugMessage('GetCreatorType -> ' + Result);
end;

function TCppPackageCreator.GetExisting: Boolean;
begin
  OTADebugHelpers.WriteDebugMessage('GetExisting');
  Result := False;
end;

function TCppPackageCreator.GetFileName: string;
begin
  OTADebugHelpers.WriteDebugMessage('GetFileName');
  Result := FPackageName;
end;

function TCppPackageCreator.GetFileSystem: string;
begin
  OTADebugHelpers.WriteDebugMessage('GetFileSystem');
  Result := '';
end;

function TCppPackageCreator.GetOptionFileName: string;
begin
  OTADebugHelpers.WriteDebugMessage('GetOptionFileName');
  Result := '';
end;

function TCppPackageCreator.GetOwner: IOTAModule;
begin
  OTADebugHelpers.WriteDebugMessage('GetOwner');
  Result := (BorlandIDEServices as IOTAModuleServices).MainProjectGroup;
end;

function TCppPackageCreator.GetProjectPersonality: string;
begin
  OTADebugHelpers.WriteDebugMessage('GetProjectPersonality');
  Result := ToolsAPI.sCBuilderPersonality;
end;

function TCppPackageCreator.GetShowSource: Boolean;
begin
  OTADebugHelpers.WriteDebugMessage('GetShowSource');
  Result := True;
end;

function TCppPackageCreator.GetUnnamed: Boolean;
begin
  OTADebugHelpers.WriteDebugMessage('GetUnnamed');
  Result := True;
end;

procedure TCppPackageCreator.NewDefaultModule;
begin
  OTADebugHelpers.WriteDebugMessage('NewDefaultModule');
  // Deprecated; do nothing.
end;

procedure TCppPackageCreator.NewDefaultProjectModule(const Project: IOTAProject);
var
  S: TResourceStream;
  TargetFile: string;
begin
  S := TResourceStream.Create(HInstance, SCppResFileResourceName, RT_RCDATA);
  try
    TargetFile := ExpandFileName(ChangeFileExt(Project.FileName, '.res'));
    WriteDebugMessage('  TargetFile = ' + TargetFile);
    if not FileExists(TargetFile) then
      S.SaveToFile(TargetFile);
    Project.AddFile(TargetFile, False);
  finally
    S.Free;
  end;
  OTADebugHelpers.WriteDebugMessage('NewDefaultProjectModule(' + Project.FileName + ')');
end;

function TCppPackageCreator.NewOptionSource(const ProjectName: string): IOTAFile;
begin
  OTADebugHelpers.WriteDebugMessage('NewOptionSource(' + ProjectName + ')');
  // Deprecated.
  Result := nil;
end;

procedure TCppPackageCreator.NewProjectResource(const Project: IOTAProject);
begin
  OTADebugHelpers.WriteDebugMessage('NewProjectResource');
  // Already done in NewDefaultProjectModule.
end;

function TCppPackageCreator.NewProjectSource(const ProjectName: string): IOTAFile;
begin
  OTADebugHelpers.WriteDebugMessage('NewProjectSource(' + ProjectName + ')');
  Result := TCppOTAFile.Create;
end;

{ TCppOTAFile }

function TCppOTAFile.GetAge: TDateTime;
begin
  OTADebugHelpers.WriteDebugMessage('IOTAFile.GetAge');
  Result := -1;
end;

function TCppOTAFile.GetSource: string;
var
  DividerLine: string;
  DividerLineBreak: string;
  IniFile: TIniFile;
begin
  OTADebugHelpers.WriteDebugMessage('IOTAFile.GetSource');
  DividerLine := SDefaultDividerLine;
  DividerLineBreak := SDefaultDividerLineBreak;
  WriteDebugMessage('bcb.bcf is ' + BinDirectory + 'bcb.bcf');
  if FileExists(BinDirectory + 'bcb.bcf') then
  begin
    IniFile := TIniFile.Create(BinDirectory + 'bcb.bcf');
    DividerLine := IniFile.ReadString(SCodeFormattingSection, SDividerLine,
                     SDefaultDividerLine);
    DividerLineBreak := IniFile.ReadString(SCodeFormattingSection, SDividerLineBreak,
                          SDefaultDividerLineBreak);
  end;
  if DividerLine <> '' then
    DividerLine := DividerLine +  CRLF;
  if DividerLineBreak <> '' then
    DividerLineBreak := DividerLineBreak + CRLF;
  Result := Format(SCppProjectSource, [DividerLine, DividerLineBreak]);
end;
{$IFEND}

end.

